/*
 *
 * Copyright © 2007 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 */
package examples.cityguide;

import java.io.IOException;

import java.util.Enumeration;
import java.util.Hashtable;

import javax.microedition.lcdui.Image;


/**
 * Lets you load image from resource and stores it in cache for
 * later usage
 */
public class ImageManager {
    private static ImageManager im = null;
    private static Hashtable imageCache = null;

    private ImageManager() {
        imageCache = new Hashtable();
    }

    public static ImageManager getInstance() {
        if (im == null) {
            im = new ImageManager();
        }

        return im;
    }

    /**
     * Load image from resource and store it
     * in cache.
     */
    public Image getImage(String name) {
        Image image = null;

        try {
            if (null == (image = (Image)imageCache.get(name))) {
                image = Image.createImage("/" + name + ".png");
                imageCache.put(name, image);
            }
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }

        return image;
    }

    /**
     * Batch load images into cache
     */
    public void loadImagesCache(String[] names) {
        for (int i = 0; i < names.length; i++) {
            try {
                if (names[i] != null) {
                    imageCache.put(names[i], Image.createImage("/" + names[i] + ".png"));
                }
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }
    }

    /**
     * Batch load images into cache
     */
    public void loadImagesCache(Enumeration e) {
        for (; e.hasMoreElements();) {
            try {
                String name = (String)e.nextElement();
                imageCache.put(name, Image.createImage("/" + name + ".png"));
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }
    }
}
